@@ -14,19 +14,24 @@ class EditDialog : public QDialog
     Q_OBJECT
 
 public:
-    explicit EditDialog(QWidget* parent = 0);
+    explicit EditDialog(QWidget* parent = 0, bool forUseInDockWidget = false);
     ~EditDialog();
 
-public:
     int getCurrentCol() { return curCol; }
     int getCurrentRow() { return curRow; }
 
 public slots:
     virtual void reset();
     virtual void loadText(const QByteArray& data, int row, int col);
+    virtual void setFocus();
+    virtual void reject();
+    virtual void allowEditing(bool on);
+
+protected:
+    virtual void closeEvent(QCloseEvent* ev);
+    virtual void showEvent(QShowEvent* ev);
 
 private slots:
-    virtual void closeEvent(QCloseEvent*);
     virtual void importData();
     virtual void exportData();
     virtual void clearData();
@@ -42,6 +47,7 @@ signals:
 
 private:
     Ui::EditDialog* ui;
+    bool useInDock;
     QHexEdit* hexEdit;
     QByteArray oldData;
     int curCol;
