@@ -9,9 +9,49 @@ namespace sqlb {
 
 QStringList Field::Datatypes = QStringList() << "INTEGER" << "TEXT" << "BLOB" << "REAL" << "NUMERIC";
 
+QString escapeIdentifier(QString id)
+{
+    return '`' + id.replace('`', "``") + '`';
+}
+
+bool ForeignKeyClause::isSet() const
+{
+    return m_override.size() || m_table.size();
+}
+
+QString ForeignKeyClause::toString() const
+{
+    if(!isSet())
+        return QString();
+
+    if(m_override.size())
+        return m_override;
+
+    QString result = escapeIdentifier(m_table);
+
+    if(m_columns.size())
+    {
+        result += "(";
+        foreach(const QString& column, m_columns)
+            result += escapeIdentifier(column) + ',';
+        result.chop(1); // Remove last comma
+        result += ")";
+    }
+
+    if(m_constraint.size())
+        result += " " + m_constraint;
+
+    return result;
+}
+
+void ForeignKeyClause::setFromString(const QString& fk)
+{
+    m_override = fk;
+}
+
 QString Field::toString(const QString& indent, const QString& sep) const
 {
-    QString str = indent + '`' + m_name + '`' + sep + m_type;
+    QString str = indent + escapeIdentifier(m_name) + sep + m_type;
     if(m_notnull)
         str += " NOT NULL";
     if(!m_defaultvalue.isEmpty())
@@ -111,7 +151,7 @@ QStringList Table::fieldList() const
 {
     QStringList sl;
 
-    foreach(FieldPtr f, m_fields) {
+    foreach(const FieldPtr& f, m_fields) {
         sl << f->toString();
     }
 
@@ -169,7 +209,7 @@ QPair<Table, bool> Table::parseSQL(const QString &sSQL)
 
 QString Table::sql() const
 {
-    QString sql = QString("CREATE TABLE `%1` (\n").arg(m_name);
+    QString sql = QString("CREATE TABLE %1 (\n").arg(escapeIdentifier(m_name));
 
     sql += fieldList().join(",\n");
 
@@ -194,8 +234,8 @@ QString Table::sql() const
     // foreign keys
     foreach(FieldPtr f, m_fields)
     {
-        if(!f->foreignKey().isEmpty())
-            sql += QString(",\n\tFOREIGN KEY(`%1`) REFERENCES %2").arg(f->name()).arg(f->foreignKey());
+        if(f->foreignKey().isSet())
+            sql += QString(",\n\tFOREIGN KEY(%1) REFERENCES %2").arg(escapeIdentifier(f->name())).arg(f->foreignKey().toString());
     }
 
     sql += "\n)";
@@ -216,9 +256,20 @@ QString identifier(antlr::RefAST ident)
        ident->getType() == Sqlite3Lexer::QUOTEDLITERAL ||
        ident->getType() == sqlite3TokenTypes::STRINGLITERAL)
     {
+        // Remember the way the identifier is quoted
+        QChar quoteChar = sident.at(0);
+
+        // Remove first and final character, i.e. the quotes
         sident.remove(0, 1);
-        sident.remove(sident.length() - 1, 1);
+        sident.chop(1);
+
+        // Replace all remaining occurences of two succeeding quote characters and replace them
+        // by a single instance. This is done because two quotes can be used as a means of escaping
+        // the quote character, thus only the visual representation has its two quotes, the actual
+        // name contains only one.
+        sident.replace(QString(quoteChar) + quoteChar, quoteChar);
     }
+
     return sident;
 }
 
@@ -372,6 +423,8 @@ Table CreateTableWalker::table()
                 break;
                 case sqlite3TokenTypes::FOREIGN:
                 {
+                    sqlb::ForeignKeyClause fk;
+
                     tc = tc->getNextSibling();  // FOREIGN
                     tc = tc->getNextSibling();  // KEY
                     tc = tc->getNextSibling();  // LPAREN
@@ -386,7 +439,28 @@ Table CreateTableWalker::table()
                     tc = tc->getNextSibling();  // RPAREN
                     tc = tc->getNextSibling();  // REFERENCES
 
-                    tab.fields().at(tab.findField(column_name))->setForeignKey(concatTextAST(tc, true));
+                    fk.setTable(identifier(tc));
+                    tc = tc->getNextSibling();       // identifier
+
+                    if(tc != antlr::nullAST && tc->getType() == sqlite3TokenTypes::LPAREN)
+                    {
+                        tc = tc->getNextSibling();  // LPAREN
+
+                        QStringList fk_cols;
+                        while(tc != antlr::nullAST && tc->getType() != sqlite3TokenTypes::RPAREN)
+                        {
+                            if(tc->getType() != sqlite3TokenTypes::COMMA)
+                                fk_cols.push_back(identifier(tc));
+                            tc = tc->getNextSibling();
+                        }
+                        fk.setColumns(fk_cols);
+
+                        tc = tc->getNextSibling();  // RPAREN
+                    }
+
+                    fk.setConstraint(concatTextAST(tc, true));
+
+                    tab.fields().at(tab.findField(column_name))->setForeignKey(fk);
                 }
                 break;
                 default:
@@ -423,7 +497,7 @@ void CreateTableWalker::parsecolumn(FieldPtr& f, antlr::RefAST c)
     bool unique = false;
     QString defaultvalue;
     QString check;
-    QString foreignKey;
+    sqlb::ForeignKeyClause foreignKey;
 
     colname = columnname(c);
     c = c->getNextSibling(); //type?
@@ -491,7 +565,27 @@ void CreateTableWalker::parsecolumn(FieldPtr& f, antlr::RefAST c)
         case sqlite3TokenTypes::REFERENCES:
         {
             con = con->getNextSibling();    // REFERENCES
-            foreignKey = concatTextAST(con, true);
+
+            foreignKey.setTable(identifier(con));
+            con = con->getNextSibling();    // identifier
+
+            if(con != antlr::nullAST && con->getType() == sqlite3TokenTypes::LPAREN)
+            {
+                con = con->getNextSibling();    // LPAREN
+
+                QStringList fk_cols;
+                while(con != antlr::nullAST && con->getType() != sqlite3TokenTypes::RPAREN)
+                {
+                    if(con->getType() != sqlite3TokenTypes::COMMA)
+                        fk_cols.push_back(identifier(con));
+                    con = con->getNextSibling();
+                }
+                foreignKey.setColumns(fk_cols);
+
+                con = con->getNextSibling();    // RPAREN
+            }
+
+            foreignKey.setConstraint(concatTextAST(con, true));
         }
         break;
         default:
